/*
 * This is system cordova_plugin (TV specific API).
 * Apache License (2004). See http://www.apache.org/licenses/LICENSE-2.0
 *
 * Copyright (c) 2014, LG Electronics, Inc.
 */


/**
 * This represents the power API itself, and provides a global namespace for operating power service.
 * @class
 */
Power = (function() {

    var Power, service;

    function log(msg) {
        ////console.log
    }

    //Browser
    if (typeof window === 'object') {
        cordova.define('cordova/plugin/power', function(require, exports, module) { // jshint ignore:line

            Power = function() {};

            if (window.PalmSystem) { // jshint ignore:line
                log("Window.PalmSystem Available");
                service = require('cordova/plugin/webos/service');
            } else {
                service = {
                    Request: function(uri, params) {
                        log(uri + " invoked. But I am a dummy because PalmSystem is not available");

                        if (typeof params.onFailure === 'function') {
                            params.onFailure({
                                returnValue: false,
                                errorText: "PalmSystem Not Available. Cordova is not installed?"
                            });
                        }
                    }
                };
            }

            module.exports = Power;
        });

        Power = cordova.require('cordova/plugin/power'); // jshint ignore:line
    }
    //NodeJS
    else {
        Power = function(serviceInstance) {
            service = serviceInstance;
            service.Request = function(serviceUri, reqParams) {
                var commandURI = serviceUri + "/" + reqParams.method;
                var parameters = {};
                if(reqParams.hasOwnProperty("parameters") === true)
                    parameters = reqParams.parameters;

                var convertedRes = {};

                var callback = function(res) {
                    console.log("res : " + JSON.stringify(res));
                    if (res.payload.returnValue === true) {
                        convertedRes = res.payload;
                        reqParams.onSuccess(convertedRes);
                    } else {
                        convertedRes.returnValue = false;
                        convertedRes.errorCode = res.payload.errorCode;
                        convertedRes.errorText = res.payload.errorText;
                        reqParams.onFailure(convertedRes);
                    }
                }
                if (service) {
                    service.call(commandURI, parameters, callback);
                }
            };
        };

        module.exports = Power;
    }

    function checkErrorCodeNText(result, errorCode, errorText) {

        if (result.errorCode === undefined || result.errorCode === null) {
            result.errorCode = errorCode;
        }
        if (result.errorText === undefined || result.errorText === null) {
            result.errorText = errorText;
        }
    }


    function arrayCheck(a, b){
        if( a === b ){
            return true;
        }
        if( a == null || b == null ){
            return false;
        }
        if( a.length != b.length ){
            return false;
        }

        a.sort();
        b.sort();

        for(var i = 0; i < a.length; i++) {
            if( a[i] !== b[i] ){
                return false;
            }
        }
        return true;

    }

    function convertExternal(extStr) {
        switch (extStr) {
            case "ext://hdmi:1":
                return "HDMI1";
            case "ext://hdmi:2":
                return "HDMI2";
            case "ext://hdmi:3":
                return "HDMI3";
            case "ext://dvi:1":
                return "DVI";
            case "ext://dp:1":
                return "DISPLAYPORT";
            case "ext://rgb:1":
                return "RGB";
            case "ext://ops:1":
                return "OPS";
            case "HDMI1":
                return "ext://hdmi:1";
            case "HDMI":
                return "ext://hdmi:1";
            case "HDMI2":
                return "ext://hdmi:2";
            case "HDMI3":
                return "ext://hdmi:3";
            case "DVI":
                return "ext://dvi:1";
            case "DISPLAYPORT":
                return "ext://dp:1";
            case "RGB":
                return "ext://rgb:1";
            case "OPS":
                return "ext://ops:1";
            case "OPS/HDMI3/DVI":
                return "ext://hdmi:3"
        }
        return null;
    }

    /**
     * @namespace Power.PowerMode
     */
    Power.PowerCommand = {
        /**
         * shutdown
         * @since 1.0
         * @constant
         */
        SHUTDOWN: "powerOff",
        /**
         * reboot
         * @since 1.0
         * @constant
         */
        REBOOT: "reboot"
    };

    /**
     * @namespace Power.DisplayMode
     */
    Power.DisplayMode = {
        /**
         * display off
         * @since 1.0
         * @constant
         */
        DISPLAY_OFF: "Screen Off",
        /**
         * display on
         * @since 1.0
         * @constant
         */
        DISPLAY_ON: "Active"
    };

    /**
     * @namespace Power.TimerWeek
     */
    Power.TimerWeek = {
        /**
         * Monday
         * @since 1.0
         * @constant
         */
        MONDAY: 1,
        /**
         * Tuesday
         * @since 1.0
         * @constant
         */
        TUESDAY: 2,
        /**
         * Wednesday
         * @since 1.0
         * @constant
         */
        WEDNESDAY: 4,
        /**
         * Thursday
         * @since 1.0
         * @constant
         */
        THURSDAY: 8,
        /**
         * Friday
         * @since 1.0
         * @constant
         */
        FRIDAY: 16,
        /**
         * Saturday
         * @since 1.0
         * @constant
         */
        SATURDAY: 32,
        /**
         * Sunday
         * @since 1.0
         * @constant
         */
        SUNDAY: 64,
        /**
         * Everyday
         * @since 1.0
         * @constant
         */
        EVERYDAY: 127
    };

    /**
     * @namespace Power.DPMSignalType
     */
    Power.DPMSignalType = {
        /**
         * CLOCK
         * @since 1.0
         * @constant
         */
        CLOCK: "clock",
        /**
         * CLOCK_WITH_DATA
         * @since 1.0
         * @constant
         */
        CLOCK_WITH_DATA: "clockAndData"
    };

    /**
     * @namespace Power.PMMode
     */
    Power.PMMode = {
        /**
         * PowerOff
         * @since 1.4
         * @constant
         */
        PowerOff: "powerOff",
        /**
         * SustainAspectRatio
         * @since 1.4
         * @constant
         */
        SustainAspectRatio: "sustainAspectRatio",
        /**
         * ScreenOff
         * @since 1.4
         * @constant
         */
        ScreenOff: "screenOff",
        /**
         * ScreenOffAlways
         * @since 1.4
         * @constant
         */
        ScreenOffAlways: "screenOffAlways",
        /**
         * ScreenOffBacklight (Only for outdoor model)
         * @since 1.4.1
         * @constant
         */
        ScreenOffBacklight: "screenOffBacklight"
    };

    /**
     * Gets power status
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return <p>{Object} </p>
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>wakeOnLan</th><th>Boolean</th><th>true : enabled / false : disabled </th></tr>
     *       <tr class="odd"><th>displayMode</th><th>String</th><th><a href="Power.DisplayMode.html#.DISPLAY_ON">Power.DisplayMode.DISPLAY_ON</a> | <a href="Power.DisplayMode.html#.DISPLAY_OFF">Power.DisplayMode.DISPLAY_OFF</a> </th></tr>
     *       <tr><th>allOnTimer</th><th>Boolean</th><th>true : enabled / false : disabled <br>this spec has been removed since v1.6</b></th></tr>
     *       <tr class="odd"><th>allOffTimer</th><th>Boolean</th><th>true : enabled / false : disabled <br>this spec has been removed since v1.6</th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getPowerStatus () {
     *   function successCb(cbObject) {
     *      console.log("cbObject : " + JSON.stringify(cbObject));
     *      console.log("wakeOnLan : " + cbObject.wakeOnLan);
     *      console.log("displayMode : " + cbObject.displayMode);
     *
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.getPowerStatus(successCb, failureCb);
     * }
     * @since 1.0
     * @since 1.6 remove allOffTimer, allOnTimer
     * @see
     * <a href="Power%23setWakeOnLan.html">Power.setWakeOnLan()</a>,
     * <a href="Power%23setDisplayMode.html">Power.setDisplayMode()</a><br>
     */
    Power.prototype.getPowerStatus = function(successCallback, errorCallback) {

        log("getPowerStatus");

        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method: "getPowerStatus",
            parameters: {},
            onSuccess: function(result) {
                log("getPowerStatus: On onSuccess");
                delete result.returnValue;
                if (typeof successCallback === 'function') {
                    successCallback(result);
                }
            },
            onFailure: function(result) {
                log("getPowerStatus: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGPS", "Power.getPowerStatus returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.getPowerStatus Done");

    };

    /**
     * Controls all "on timer". This API enables/disables all "on timer" at once. Values of each "on timer" which was set by Power.addOnTimer() are not changed when option.clearOnTimer is false.
     *
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>allOnTimer</th><th>Boolean</th><th>true : enabled / false : disabled </th><th>required</th></tr>
     *       <tr class="odd"><th>clearOnTimer</th><th>Boolean</th><th>true : removes all "on Timer" / false : do nothing. (default) </th><th>optional</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function enableAllOnTimer () {
     *   var options = {
     *      allOnTimer : true
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.enableAllOnTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.reserveOnOffTimer instead.<br>
     * @since 1.0
     * @since 1.3 optons.clearOnTimer
     * @see
     * <a href="Power%23getPowerStatus.html">Power.getPowerStatus()</a><br>
     */
    Power.prototype.enableAllOnTimer = function(successCallback, errorCallback, options) {

        log("enableAllOnTimer: " + JSON.stringify(options));

        var op = null;
        switch (options.allOnTimer) {
            case true:
                op = "on";
                break;
            case false:
                op = "off";
                break;

            default:
                if (typeof errorCallback === 'function') {
                    var result = {};
                    checkErrorCodeNText(result, "PEAOT", "Power.enableAllOnTimer returns failure. Invalid option value.");
                    errorCallback(result);
                }
                return;
        }

        // webOS 4.0 or above
        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "set",
            parameters: {
                category: "time",
                settings: {
                    "onTimerEnable": op
                }
            },
            onSuccess: function() {
                if (options.clearOnTimer === true) {
                    service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                        method: "get",
                        parameters: {
                            category: "commercial",
                            keys: ["onOffTimeSchedule"]
                        },
                        onSuccess: function(result) {
                            if (result.returnValue === true) {
                                let i;
                                // onOffTimeSchedule : [ oneTime : { days : [] , hour, minute } , oneTime : {} , ... ]
                                let onOffTimeList = {};
                                onOffTimeList.onTime = [];
                                onOffTimeList.offTime = [];
                                let oneTime = {};
                                let indexOff = (result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);
                                for(i=0; i<indexOff; i++){
                                    if(result.settings.onOffTimeSchedule.offTime[i].day !== undefined && result.settings.onOffTimeSchedule.offTime[i].day !== null){
                                        oneTime.day = result.settings.onOffTimeSchedule.offTime[i].day;
                                    }
                                    if(result.settings.onOffTimeSchedule.offTime[i].hour !== undefined && result.settings.onOffTimeSchedule.offTime[i].hour !== null){
                                        oneTime.hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                                    }
                                    if(result.settings.onOffTimeSchedule.offTime[i].minute !== undefined && result.settings.onOffTimeSchedule.offTime[i].minute !== null){
                                        oneTime.minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                                    }
                                    onOffTimeList.offTime.push(oneTime);
                                    oneTime ={};
                                }

                                service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                                    method: "set",
                                    parameters: {
                                        category: "commercial",
                                        settings: { "onOffTimeSchedule" : onOffTimeList }
                                    },
                                    onSuccess: function() {
                                        log("enableAllOnTimer: On Success 2");

                                        if (typeof successCallback === 'function') {
                                            successCallback();
                                        }
                                    },
                                    onFailure: function(result) {
                                        log("enableAllOnTimer: On Failure 2");
                                        delete result.returnValue;
                                        if (typeof errorCallback === 'function') {
                                            checkErrorCodeNText(result, "PAOT", "Power.enableAllOnTimer returns failure.");
                                            errorCallback(result);
                                        }
                                        return;
                                    }
                                });
                            } else {
                                if (typeof successCallback === 'function') {
                                    log("enableAllOnTimer: On Success");
                                    successCallback();
                                }
                            }
                        },
                        onFailure: function(result) {
                            delete result.returnValue;
                            if (typeof errorCallback === 'function') {
                                log("enableAllOnTimer: On Failure");
                                checkErrorCodeNText(result, "PEAOT", "Power.enableAllOnTimer returns failure.");
                                errorCallback(result);
                            }
                        }
                    });
                } else {
                    if (typeof successCallback === 'function') {
                        log("enableAllOnTimer: On Success");
                        successCallback();
                    }
                }
            },
            onFailure: function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    log("enableAllOnTimer: On Failure");
                    checkErrorCodeNText(result, "PEAOT", "Power.enableAllOnTimer returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.enableAllOnTimer Done");
    };

    /**
     * Controls all "off timer". This API enables/disables all "off timer" at once. Values of each "off timer" which was set by Power.addOffTimer() are not changed when options.clearOffTimer is false.
     *
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>allOffTimer</th><th>Boolean</th><th>true : enabled / false : disabled </th><th>required</th></tr>
     *       <tr class="odd"><th>clearOffTimer</th><th>Boolean</th><th>true : removes all "off Timer" / false : do nothing. (default) </th><th>optional</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function enableAllOffTimer () {
     *   var options = {
     *      allOffTimer : true
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.enableAllOffTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.reserveOnOffTimer instead.<br>
     * @since 1.0
     * @since 1.3 options.clearOffTimer
     * @see
     * <a href="Power%23getPowerStatus.html">Power.getPowerStatus()</a><br>
     */
    Power.prototype.enableAllOffTimer = function(successCallback, errorCallback, options) {

        log("enableAllOffTimer: " + JSON.stringify(options));

        var op = null;

        switch (options.allOffTimer) {
            case true:
                op = "on";
                break;
            case false:
                op = "off";
                break;
            default:
                if (typeof errorCallback === 'function') {
                    var result = {};
                    checkErrorCodeNText(result, "PEAOT", "Power.enableAllOffTimer returns failure. Invalid option value.");
                    errorCallback(result);
                }
                return;
        }

        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "set",
            parameters: {
                category: "time",
                settings: {
                    "offTimerEnable": op
                }
            },
            onSuccess: function() {
                if (options.clearOffTimer === true) {
                    service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                        method: "get",
                        parameters: {
                            category: "commercial",
                            keys: ["onOffTimeSchedule"]
                        },
                        onSuccess: function(result) {
                            if (result.returnValue === true) {
                                let i;
                                let onOffTimeList = {};
                                onOffTimeList.onTime = [];
                                onOffTimeList.offTime = [];
                                let oneTime = {};
                                let indexOn = (result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);
                                for(i=0; i<indexOn; i++){
                                    if(result.settings.onOffTimeSchedule.onTime[i].day !== undefined && result.settings.onOffTimeSchedule.onTime[i].day !== null){
                                        oneTime.day = result.settings.onOffTimeSchedule.onTime[i].day;
                                    }
                                    if(result.settings.onOffTimeSchedule.onTime[i].hour !== undefined && result.settings.onOffTimeSchedule.onTime[i].hour !== null){
                                        oneTime.hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                                    }
                                    if(result.settings.onOffTimeSchedule.onTime[i].minute !== undefined && result.settings.onOffTimeSchedule.onTime[i].minute !== null){
                                        oneTime.minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                                    }
                                    onOffTimeList.onTime.push(oneTime);
                                    oneTime ={};
                                }

                                service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                                    method: "set",
                                    parameters: {
                                        category: "commercial",
                                        settings: { "onOffTimeSchedule" : onOffTimeList }
                                    },
                                    onSuccess: function() {
                                        log("enableAllOffTimer: On Success 2");

                                        if (typeof successCallback === 'function') {
                                            successCallback();
                                        }
                                    },
                                    onFailure: function(result) {
                                        log("enableAllOffTimer: On Failure 2");
                                        delete result.returnValue;
                                        if (typeof errorCallback === 'function') {
                                            checkErrorCodeNText(result, "PAOT", "Power.enableAllOffTimer returns failure.");
                                            errorCallback(result);
                                        }
                                        return;
                                    }
                                });
                            } else {
                                if (typeof successCallback === 'function') {
                                    log("enableAllOffTimer: On Success");
                                    successCallback();
                                }
                            }
                        },
                        onFailure: function(result) {
                            delete result.returnValue;
                            if (typeof errorCallback === 'function') {
                                log("enableAllOffTimer: On Failure");
                                checkErrorCodeNText(result, "PEAOT", "Power.enableAllOffTimer returns failure.");
                                errorCallback(result);
                            }
                        }
                    });
                } else {
                    if (typeof successCallback === 'function') {
                        log("enableAllOffTimer: On Success");
                        successCallback();
                    }
                }
            },
            onFailure: function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    log("enableAllOffTimer: On Failure");
                    checkErrorCodeNText(result, "PEAOT", "Power.enableAllOffTimer returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.enableAllOffTimer Done");

    };

    /**
     * Enables or disables 'wake on LAN'.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>wakeOnLan</th><th>Boolean</th><th>true : enabled / false : disabled </th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function enableWakeOnLan () {
     *   var options = {
     *      wakeOnLan : true
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.enableWakeOnLan(successCb, failureCb, options);
     * }
     * @since 1.0
     * @see
     * <a href="Power%23getPowerStatus.html">Power.getPowerStatus()</a><br>
     */
    Power.prototype.enableWakeOnLan = function(successCallback, errorCallback, options) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "enableWakeOnLan",
            parameters : options,
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback();
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PEWO", "Power.enableWakeOnLan returns failure.");
                    errorCallback(result);
                }
            }
        });
    };


    /**
     * Adds 'on timer'.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>hour</th><th>Number</th><th>hour (0~23)</th><th>required</th></tr>
     *       <tr class="odd"><th>minute</th><th>Number</th><th>minute (0~59)</th><th>required</th></tr>
     *       <tr><th>week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> <br>To use it on multiple days of week, set the sum of week. <br>For example, 1 + 64 (Power.TimerWeek.Monday + Power.TimerWeek.Sunday) means that this timer works on every Monday and Sunday.</th><th>required</th></tr>
     *       <tr class="odd"><th>inputSource</th><th>String</th><th>Define the input source. ext://[externalInput]:[portNum]. (eg: "ext://hdmi:1")</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function addOnTimer () {
     *   var options = {
     *      hour : 9,
     *      minute : 0,
     *      week : Power.TimerWeek.MONDAY + Power.TimerWeek.FRIDAY,
     *      inputSource : "ext://hdmi:1"
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.addOnTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.reserveOnOffTimer instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23getOnTimerList.html">Power.getOnTimerList()</a>,
     * <a href="InputSource.%23getInputSourceStatus.html">InputSource.getInputSourceStatus()</a><br>
     */
    Power.prototype.addOnTimer = function(successCallback, errorCallback, options) {
        log("addOnTimer: " + JSON.stringify(options));

        if (options.hour === undefined || isNaN(options.hour) || typeof options.hour !== 'number' || options.hour < 0 || options.hour > 23 ||
        options.minute === undefined || isNaN(options.minute) || typeof options.minute !== 'number' || options.minute < 0 || options.minute > 59 ||
        options.week === undefined || isNaN(options.week) || typeof options.week !== 'number' || options.week <= 0 || options.week > 127) {

            if (typeof errorCallback === 'function') {
                var result = {};
                checkErrorCodeNText(result, "PAOT", "Power.addOnTimer returns failure. invalid parameters or out of range.");
                errorCallback(result);
            }
            return;
        }

        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {
                if (result.returnValue === true) {
                    let i;
                    // onOffTimeSchedule : [ oneTime : { days : [] , hour, minute } , oneTime : {} , ... ]
                    let onOffTimeList = {};
                    onOffTimeList.onTime = [];
                    onOffTimeList.offTime = [];
                    let days = [];
                    let oneTime ={};
                    let counter=1;
                    while(options.week !== 0){
                        if(options.week & 1){
                            console.log('AddonTimer - options.week - ' + options.week);
                            //WEBOS 3.0 mon(1), tue(2), wed(4), thu(8), fri(16), sat(32). sun(64)
                            switch(counter){
                                case 1:
                                    days.push('mon');
                                    break;
                                case 2:
                                    days.push('tue');
                                    break;
                                case 4:
                                    days.push('wed');
                                    break;
                                case 8:
                                    days.push('thu');
                                    break;
                                case 16:
                                    days.push('fri');
                                    break;
                                case 32:
                                    days.push('sat');
                                    break;
                                case 64:
                                    days.push('sun');
                                    break;
                            }
                        }
                        options.week = options.week >>> 1;
                        counter = counter << 1;
                        if(options.week === 0){
                            break;
                        }
                    }
                    oneTime.day = days;
                    oneTime.hour = options.hour;
                    oneTime.minute = options.minute;
                    onOffTimeList.onTime.push(oneTime);
                    // onTimer set from user input
                    oneTime ={};
                    // getSystemSettings input to set as a list.
                    let index = (result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);
                    for(i=0; i<index; i++){

                        if(result.settings.onOffTimeSchedule.onTime[i].day !== undefined && result.settings.onOffTimeSchedule.onTime[i].day !== null){
                            oneTime.day = result.settings.onOffTimeSchedule.onTime[i].day;
                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].hour !== undefined && result.settings.onOffTimeSchedule.onTime[i].hour !== null){
                            oneTime.hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].minute !== undefined && result.settings.onOffTimeSchedule.onTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                        }

                        onOffTimeList.onTime.push(oneTime);
                        oneTime ={};
                    }

                    let indexOff = (result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);
                    for(i=0; i<indexOff; i++){
                        if(result.settings.onOffTimeSchedule.offTime[i].day !== undefined && result.settings.onOffTimeSchedule.offTime[i].day !== null){
                            oneTime.day = result.settings.onOffTimeSchedule.offTime[i].day;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].hour !== undefined && result.settings.onOffTimeSchedule.offTime[i].hour !== null){
                            oneTime.hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].minute !== undefined && result.settings.onOffTimeSchedule.offTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                        }
                        onOffTimeList.offTime.push(oneTime);
                        oneTime ={};
                    }

                    service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                        method: "set",
                        parameters: {
                            category: "commercial",
                            settings: { "onOffTimeSchedule" : onOffTimeList }
                        },
                        onSuccess: function() {
                            log("addOnTimer: On Success 2");

                            if (typeof successCallback === 'function') {
                                successCallback();
                            }
                        },
                        onFailure: function(result) {
                            log("addOnTimer: On Failure 2");
                            delete result.returnValue;
                            if (typeof errorCallback === 'function') {
                                checkErrorCodeNText(result, "PAOT", "Power.addOnTimer returns failure.");
                                errorCallback(result);
                            }
                            return;
                        }
                    });
                }
            },
            onFailure: function(result) {
                log("addOnTimer: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PAOT", "Power.addOnTimer returns failure.");
                    errorCallback(result);
                }
                return;
            }
        });

        log("Power.addOnTimer Done");
    };

    /**
     * Deletes 'on timer'. The timer in the list that matches the parameter will be removed.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>hour</th><th>Number</th><th>hour (0~23)</th><th>required</th></tr>
     *       <tr class="odd"><th>minute</th><th>Number</th><th>minute (0~59)</th><th>required</th></tr>
     *       <tr><th>week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> <br>To use it on multiple day of week, set the sum of week. <br>For example, 1 + 64 (Power.TimerWeek.Monday + Power.TimerWeek.Sunday) means that this timer works on every Monday and Sunday.</th><th>required</th></tr>
     *       <tr class="odd"><th>inputSource</th><th>String</th><th>Define the input source. ext://[externalInput]:[portNum]. (eg: "ext://hdmi:1")</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function deleteOnTimer () {
     *   var options = {
     *      hour : 9,
     *      minute : 0,
     *      week : Power.TimerWeek.MONDAY + Power.TimerWeek.FRIDAY,
     *      inputSource : "ext://hdmi:1"
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.deleteOnTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.cancelOnOffTimer instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23getOnTimerList.html">Power.getOnTimerList()</a>,
     * <a href="InputSource.%23getInputSourceStatus.html">InputSource.getInputSourceStatus()</a><br>
     */
    Power.prototype.deleteOnTimer = function(successCallback, errorCallback, options) {
        log("deleteOnTimer: " + JSON.stringify(options));

        if (options.hour === undefined || isNaN(options.hour) || typeof options.hour !== 'number' || options.hour < 0 || options.hour > 23 ||
        options.minute === undefined || isNaN(options.minute) || typeof options.minute !== 'number' || options.minute < 0 || options.minute > 59 ||
        options.week === undefined || isNaN(options.week) || typeof options.week !== 'number' || options.week < 0 || options.week > 127) {

            if (typeof errorCallback === 'function') {
                var result = {};
                checkErrorCodeNText(result, "PAOT", "Power.deleteOnTimer returns failure. invalid parameters or out of range.");
                errorCallback(result);
            }
            return;
        }

        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {
                if (result.returnValue === true) {
                    let i;
                    // onOffTimeSchedule : [ oneTime : { days : [] , hour, minute } , oneTime : {} , ... ]
                    let onOffTimeList = {};
                    onOffTimeList.onTime = [];
                    onOffTimeList.offTime = [];
                    let days = [];
                    let oneTimeToDel ={};
                    let oneTime = {};
                    let counter = 1;
                    while(options.week !== 0){
                        if(options.week & 1){
                            // console.log('AddonTimer - options.week - ' + options.week);
                            //WEBOS 3.0 mon(1), tue(2), wed(4), thu(8), fri(16), sat(32). sun(64)
                            switch(counter){
                                case 1:
                                    days.push('mon');
                                    break;
                                case 2:
                                    days.push('tue');
                                    break;
                                case 4:
                                    days.push('wed');
                                    break;
                                case 8:
                                    days.push('thu');
                                    break;
                                case 16:
                                    days.push('fri');
                                    break;
                                case 32:
                                    days.push('sat');
                                    break;
                                case 64:
                                    days.push('sun');
                                    break;
                            }
                        }
                        options.week = options.week >>> 1;
                        counter = counter << 1;
                        if(options.week === 0){
                            break;
                        }
                    }
                    counter = 0;
                    oneTimeToDel.day = days;
                    oneTimeToDel.hour = options.hour;
                    oneTimeToDel.minute = options.minute;
                    let index = (result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);
                    let matched = false;
                    for(i=0; i<index; i++){

                        if(result.settings.onOffTimeSchedule.onTime[i].day !== undefined && result.settings.onOffTimeSchedule.onTime[i].day !== null){
                            if(arrayCheck(result.settings.onOffTimeSchedule.onTime[i].day, oneTimeToDel.day)) {
                                //Array compare
                                counter++;
                            } else {
                                oneTime.day = result.settings.onOffTimeSchedule.onTime[i].day;
                            }

                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].hour !== undefined && result.settings.onOffTimeSchedule.onTime[i].hour !== null){
                            if( result.settings.onOffTimeSchedule.onTime[i].hour === oneTimeToDel.hour){
                                counter++;
                            } else {
                                oneTime.hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                            }
                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].minute !== undefined && result.settings.onOffTimeSchedule.onTime[i].minute !== null){
                            if( result.settings.onOffTimeSchedule.onTime[i].minute === oneTimeToDel.minute){
                                counter++;
                            } else {
                                oneTime.minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                            }
                        }

                        if(counter !== 3){
                            onOffTimeList.onTime.push(oneTime);
                        } else {
                            matched = true;
                        }
                        counter = 0;
                        oneTime = {};
                    }

                    let indexOff = (result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);
                    for(i=0; i<indexOff; i++){
                        if(result.settings.onOffTimeSchedule.offTime[i].day !== undefined && result.settings.onOffTimeSchedule.offTime[i].day !== null){
                            oneTime.day = result.settings.onOffTimeSchedule.offTime[i].day;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].hour !== undefined && result.settings.onOffTimeSchedule.offTime[i].hour !== null){
                            oneTime.hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].minute !== undefined && result.settings.onOffTimeSchedule.offTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                        }
                        onOffTimeList.offTime.push(oneTime);
                        oneTime ={};
                    }

                    if(matched){
                        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                            method: "set",
                            parameters: {
                                category: "commercial",
                                settings: { "onOffTimeSchedule" : onOffTimeList }
                            },
                            onSuccess: function() {
                                log("deleteOnTimer: On Success 2");

                                if (typeof successCallback === 'function') {
                                    successCallback();
                                }
                            },
                            onFailure: function(result) {
                                log("deleteOnTimer: On Failure 2");
                                delete result.returnValue;
                                if (typeof errorCallback === 'function') {
                                    checkErrorCodeNText(result, "PAOT", "Power.deleteOnTimer returns failure.");
                                    errorCallback(result);
                                }
                                return;
                            }
                        });
                    } else {
                        let result = {};
                        checkErrorCodeNText(result, "PAOT", "Power.deleteOnTimer returns failure. / No matched found");
                        errorCallback(result);
                    }

                }
            },
            onFailure: function(result) {
                log("deleteOnTimer: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PAOT", "Power.deleteOnTimer returns failure.");
                    errorCallback(result);
                }
                return;
            }
        });

        log("Power.deleteOnTimer Done");
    };

    /**
     * Adds 'off timer'.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>hour</th><th>Number</th><th>hour (0~23)</th><th>required</th></tr>
     *       <tr class="odd"><th>minute</th><th>Number</th><th>minute (0~59)</th><th>required</th></tr>
     *       <tr><th>week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> <br>To use it on multiple days of week, set the sum of week. <br>For example, 1 + 64 (Power.TimerWeek.Monday + Power.TimerWeek.Sunday) means that this timer works on every Monday and Sunday.</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function addOffTimer () {
     *   var options = {
     *      hour : 9,
     *      minute : 0,
     *      week : Power.TimerWeek.MONDAY + Power.TimerWeek.FRIDAY
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.addOffTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.reserveOnOffTimer instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23getOffTimerList.html">Power.getOffTimerList()</a><br>
     */
    Power.prototype.addOffTimer = function(successCallback, errorCallback, options) {
        if (options.hour === undefined || isNaN(options.hour) || typeof options.hour !== 'number' || options.hour < 0 || options.hour > 23 ||
            options.minute === undefined || isNaN(options.minute) || typeof options.minute !== 'number' || options.minute < 0 || options.minute > 59 ||
            options.week === undefined || isNaN(options.week) || typeof options.week !== 'number' || options.week <= 0 || options.week > 127) {

            if (typeof errorCallback === 'function') {
                var result = {};
                checkErrorCodeNText(result, "PAOT", "Power.addOffTimer returns failure. Invalid parameter.");
                errorCallback(result);
            }
            return;
        }

        //1. timer enable / disable
        //2. set time information
        //3. set the number of timer.
        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {
                if (result.returnValue === true) {
                    let i;
                    // onOffTimeSchedule : [ oneTime : { days : [] , hour, minute } , oneTime : {} , ... ]
                    let onOffTimeList = {};
                    onOffTimeList.onTime = [];
                    onOffTimeList.offTime = [];
                    let days = [];
                    let oneTime ={};
                    let counter=1;
                    while(options.week !== 0){
                        if(options.week & 1){
                            //WEBOS 3.0 mon(1), tue(2), wed(4), thu(8), fri(16), sat(32). sun(64)
                            switch(counter){
                                case 1:
                                    days.push('mon');
                                    break;
                                case 2:
                                    days.push('tue');
                                    break;
                                case 4:
                                    days.push('wed');
                                    break;
                                case 8:
                                    days.push('thu');
                                    break;
                                case 16:
                                    days.push('fri');
                                    break;
                                case 32:
                                    days.push('sat');
                                    break;
                                case 64:
                                    days.push('sun');
                                    break;
                            }
                        }
                        options.week = options.week >>> 1;
                        counter = counter << 1;
                        if(options.week === 0){
                            break;
                        }
                    }
                    oneTime.day = days;
                    oneTime.hour = options.hour;
                    oneTime.minute = options.minute;
                    onOffTimeList.offTime.push(oneTime);
                    //user input onTimer set
                    oneTime ={};
                    // getSystemSettings input to set as a list.
                    let index = (result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);
                    for(i=0; i<index; i++){

                        if(result.settings.onOffTimeSchedule.onTime[i].day !== undefined && result.settings.onOffTimeSchedule.onTime[i].day !== null){
                            oneTime.day = result.settings.onOffTimeSchedule.onTime[i].day;
                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].hour !== undefined && result.settings.onOffTimeSchedule.onTime[i].hour !== null){
                            oneTime.hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                        }
                        if(result.settings.onOffTimeSchedule.onTime[i].minute !== undefined && result.settings.onOffTimeSchedule.onTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                        }

                        onOffTimeList.onTime.push(oneTime);
                        oneTime ={};
                    }

                    let indexOff = (result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);
                    for(i=0; i<indexOff; i++){
                        if(result.settings.onOffTimeSchedule.offTime[i].day !== undefined && result.settings.onOffTimeSchedule.offTime[i].day !== null){
                            oneTime.day = result.settings.onOffTimeSchedule.offTime[i].day;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].hour !== undefined && result.settings.onOffTimeSchedule.offTime[i].hour !== null){
                            oneTime.hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].minute !== undefined && result.settings.onOffTimeSchedule.offTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                        }
                        onOffTimeList.offTime.push(oneTime);
                        oneTime ={};
                    }

                    service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                        method: "set",
                        parameters: {
                            category: "commercial",
                            settings: { "onOffTimeSchedule" : onOffTimeList }
                        },
                        onSuccess: function() {
                            log("addOffTimer: On Success 2");

                            if (typeof successCallback === 'function') {
                                successCallback();
                            }
                        },
                        onFailure: function(result) {
                            log("addOffTimer: On Failure 2");
                            delete result.returnValue;
                            if (typeof errorCallback === 'function') {
                                checkErrorCodeNText(result, "PAOT", "Power.addOffTimer returns failure.");
                                errorCallback(result);
                            }
                            return;
                        }
                    });
                }
            },
            onFailure: function(result) {
                log("addOffTimer: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PAOT", "Power.addOffTimer returns failure.");
                    errorCallback(result);
                }
                return;
            }
        });

        log("Power.addOffTimer Done");
    };

    /**
     * Deletes 'off timer'. The timer in the list that matches the parameter will be removed.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>hour</th><th>Number</th><th>hour (0~23)</th><th>required</th></tr>
     *       <tr class="odd"><th>minute</th><th>Number</th><th>minute (0~59)</th><th>required</th></tr>
     *       <tr><th>week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> <br>To use it on multiple day of week, set the sum of week. <br>For example, 1 + 64 (Power.TimerWeek.Monday + Power.TimerWeek.Sunday) means that this timer works on every Monday and Sunday.</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function deleteOffTimer () {
     *   var options = {
     *      hour : 9,
     *      minute : 0,
     *      week : Power.TimerWeek.MONDAY + Power.TimerWeek.FRIDAY
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.deleteOffTimer(successCb, failureCb, options);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.cancelOnOffTimer instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23getOffTimerList.html">Power.getOffTimerList()</a>,
     * <a href="InputSource.%23getInputSourceStatus.html">InputSource.getInputSourceStatus()</a><br>
     */
    Power.prototype.deleteOffTimer = function(successCallback, errorCallback, options) {
        log("deleteOffTimer: " + JSON.stringify(options));

        if (options.hour === undefined || isNaN(options.hour) || typeof options.hour !== 'number' || options.hour < 0 || options.hour > 23 ||
        options.minute === undefined || isNaN(options.minute) || typeof options.minute !== 'number' || options.minute < 0 || options.minute > 59 ||
        options.week === undefined || isNaN(options.week) || typeof options.week !== 'number' || options.week < 0 || options.week > 127) {

            if (typeof errorCallback === 'function') {
                var result = {};
                checkErrorCodeNText(result, "PAOT", "Power.deleteOffTimer returns failure. invalid parameters or out of range.");
                errorCallback(result);
            }
            return;
        }

        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {
                if (result.returnValue === true) {
                    let i;
                    // onOffTimeSchedule : [ oneTime : { days : [] , hour, minute } , oneTime : {} , ... ]
                    let onOffTimeList = {};
                    onOffTimeList.onTime = [];
                    onOffTimeList.offTime = [];
                    let days = [];
                    let oneTimeToDel ={};
                    let oneTime = {};
                    let counter = 1;
                    while(options.week !== 0){
                        if(options.week & 1){
                            // console.log('AddonTimer - options.week - ' + options.week);
                            //WEBOS 3.0 mon(1), tue(2), wed(4), thu(8), fri(16), sat(32). sun(64)
                            switch(counter){
                                case 1:
                                    days.push('mon');
                                    break;
                                case 2:
                                    days.push('tue');
                                    break;
                                case 4:
                                    days.push('wed');
                                    break;
                                case 8:
                                    days.push('thu');
                                    break;
                                case 16:
                                    days.push('fri');
                                    break;
                                case 32:
                                    days.push('sat');
                                    break;
                                case 64:
                                    days.push('sun');
                                    break;
                            }
                        }
                        options.week = options.week >>> 1;
                        counter = counter << 1;
                        if(options.week === 0){
                            break;
                        }
                    }
                    counter = 0;
                    oneTimeToDel.day = days;
                    oneTimeToDel.hour = options.hour;
                    oneTimeToDel.minute = options.minute;
                    // onOffTimeList.onTime.push(oneTime);
                    //user input onTimer set
                    // oneTime ={};
                    // getSystemSettings input to set as a list.
                    let index = (result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);
                    let matched = false;
                    for(i=0; i<index; i++){

                        if(result.settings.onOffTimeSchedule.onTime[i].day !== undefined && result.settings.onOffTimeSchedule.onTime[i].day !== null){
                                oneTime.day = result.settings.onOffTimeSchedule.onTime[i].day;
                        }

                        if(result.settings.onOffTimeSchedule.onTime[i].hour !== undefined && result.settings.onOffTimeSchedule.onTime[i].hour !== null){
                                oneTime.hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                        }

                        if(result.settings.onOffTimeSchedule.onTime[i].minute !== undefined && result.settings.onOffTimeSchedule.onTime[i].minute !== null){
                            oneTime.minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                        }
                        onOffTimeList.onTime.push(oneTime);
                        oneTime = {};
                    }

                    let indexOff = (result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);
                    for(i=0; i<indexOff; i++){
                        if(result.settings.onOffTimeSchedule.offTime[i].day !== undefined && result.settings.onOffTimeSchedule.offTime[i].day !== null){
                            if(arrayCheck(result.settings.onOffTimeSchedule.offTime[i].day, oneTimeToDel.day)) {
                                counter++;
                            } else {
                                oneTime.day = result.settings.onOffTimeSchedule.offTime[i].day;
                            }
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].hour !== undefined && result.settings.onOffTimeSchedule.offTime[i].hour !== null){
                            if( result.settings.onOffTimeSchedule.offTime[i].hour === oneTimeToDel.hour){
                                counter++;
                            } else {
                                oneTime.hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                            }
                        }
                        if(result.settings.onOffTimeSchedule.offTime[i].minute !== undefined && result.settings.onOffTimeSchedule.offTime[i].minute !== null){
                            if( result.settings.onOffTimeSchedule.offTime[i].minute === oneTimeToDel.minute){
                                counter++;
                            } else {
                                oneTime.minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                            }
                        }

                        if(counter !== 3){
                            onOffTimeList.offTime.push(oneTime);
                        } else {
                            matched = true;
                        }
                        counter = 0;
                        oneTime = {};
                    }

                    if(matched){
                        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
                            method: "set",
                            parameters: {
                                category: "commercial",
                                settings: { "onOffTimeSchedule" : onOffTimeList }
                            },
                            onSuccess: function() {
                                log("deleteOffTimer: On Success 2");

                                if (typeof successCallback === 'function') {
                                    successCallback();
                                }
                            },
                            onFailure: function(result) {
                                log("deleteOffTimer: On Failure 2");
                                delete result.returnValue;
                                if (typeof errorCallback === 'function') {
                                    checkErrorCodeNText(result, "PAOT", "Power.deleteOffTimer returns failure.");
                                    errorCallback(result);
                                }
                                return;
                            }
                        });
                    } else {
                        let result = {};
                        checkErrorCodeNText(result, "PAOT", "Power.deleteOffTimer returns failure. / No matched found.");
                        errorCallback(result);
                    }

                }
            },
            onFailure: function(result) {
                log("deleteOffTimer: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PAOT", "Power.deleteOffTimer returns failure.");
                    errorCallback(result);
                }
                return;
            }
        });

        log("Power.deleteOffTimer Done");
    };

    /**
     * Gets 'on timer' list.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return <p>{Object} list of timer object.</p>
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>timerList[]</th><th>Array</th><th>An array with the timer information object as its elements. </th></tr>
     *       <tr class="odd"><th>timerList[].hour</th><th>Number</th><th>hour (0~23) </th></tr>
     *       <tr><th>timerList[].minute</th><th>Number</th><th>minute (0~59) </th></tr>
     *       <tr class="odd"><th>timerList[].week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> </th></tr>
     *       <tr><th>timerList[].inputSource</th><th>String</th><th>Input source</th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getOnTimerList () {
     *   function successCb(cbObject) {
     *      var timerList = cbObject.timerList;
     *      for ( var i = 0; i < timerList.length; i++) {
     *         console.log("timerList[" + i + "] : " + JSON.stringify(timerList[i]));
     *         console.log("timerList[" + i + "].hour : " + timerList[i].hour);
     *         console.log("timerList[" + i + "].minute : " + timerList[i].minute);
     *         console.log("timerList[" + i + "].week : " + timerList[i].week);
     *         console.log("timerList[" + i + "].inputSource : " + timerList[i].inputSource);
     *      }
     *
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.getOnTimerList(successCb, failureCb);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.getAllOnOffTimers instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23setOnTimer.html">Power.setOnTimer()</a><br>
     */
    Power.prototype.getOnTimerList = function(successCallback, errorCallback) {
        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {

                if (result.returnValue === true) {

                    var cbObj = {};

                    var timerList = new Array(result.settings.onOffTimeSchedule.onTime === null || result.settings.onOffTimeSchedule.onTime === undefined ? 0 : result.settings.onOffTimeSchedule.onTime.length);

                    for (var index = 0, i = 0; i < timerList.length; i++, index++) {
                        if (result.settings.onOffTimeSchedule.onTime[i] === null || result.settings.onOffTimeSchedule.onTime[i] === undefined) {
                            continue;
                        }
                        timerList[index] = {
                            hour: 0,
                            minute: 0,
                            week: 0
                        };

                        var tempWeek = result.settings.onOffTimeSchedule.onTime[i].day;
                        var weekToNum = 0;
                        for(var j = 0; j < tempWeek.length; j++){
                            if(tempWeek[j] === 'mon'){
                                weekToNum += 1;
                            }
                            if(tempWeek[j] === 'tue'){
                                weekToNum += 2;
                            }
                            if(tempWeek[j] === 'wed'){
                                weekToNum += 4;
                            }
                            if(tempWeek[j] === 'thu'){
                                weekToNum += 8;
                            }
                            if(tempWeek[j] === 'fri'){
                                weekToNum += 16;
                            }
                            if(tempWeek[j] === 'sat'){
                                weekToNum += 32;
                            }
                            if(tempWeek[j] === 'sun'){
                                weekToNum += 64;
                            }

                        }
                        timerList[index].hour = result.settings.onOffTimeSchedule.onTime[i].hour;
                        timerList[index].minute = result.settings.onOffTimeSchedule.onTime[i].minute;
                        timerList[index].week = weekToNum;
                    }

                    cbObj.timerList = timerList;

                    if (typeof successCallback === 'function') {
                        successCallback(cbObj);
                    }
                }
            },
            onFailure: function(result) {
                log("getOnTimerList: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGOTL", "Power.getOnTimerList returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.getOnTimerList Done");

    };

    /**
     * Gets 'off timer' list.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return <p>{Object} list of timer object.</p>
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>timerList[]</th><th>Array</th><th>An array with the timer information object as its elements. </th></tr>
     *       <tr class="odd"><th>timerList[].hour</th><th>Number</th><th>hour (0~23) </th></tr>
     *       <tr><th>timerList[].minute</th><th>Number</th><th>minute (0~59) </th></tr>
     *       <tr class="odd"><th>timerList[].week</th><th>Number</th><th>week <a href="Power.TimerWeek.html#constructor">Power.TimerWeek</a> </th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getOffTimerList () {
     *   function successCb(cbObject) {
     *      var timerList = cbObject.timerList;
     *      for ( var i = 0; i < timerList.length; i++) {
     *         console.log("timerList[" + i + "] : " + JSON.stringify(timerList[i]));
     *         console.log("timerList[" + i + "].hour : " + timerList[i].hour);
     *         console.log("timerList[" + i + "].minute : " + timerList[i].minute);
     *         console.log("timerList[" + i + "].week : " + timerList[i].week);
     *      }
     *
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.getOffTimerList(successCb, failureCb);
     * }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.getAllOnOffTimers instead.<br>
     * @since 1.0
     * @see
     * <a href="Power%23setOffTimer.html">Power.setOffTimer()</a><br>
     */
    Power.prototype.getOffTimerList = function(successCallback, errorCallback) {
        log("getOffTimerList: ");

        service.Request("luna://com.webos.service.commercial.scapadapter/settings/", {
            method: "get",
            parameters: {
                category: "commercial",
                keys: ["onOffTimeSchedule"]
            },
            onSuccess: function(result) {

                if (result.returnValue === true) {

                    var cbObj = {};
                    var timerList = new Array(result.settings.onOffTimeSchedule.offTime === null || result.settings.onOffTimeSchedule.offTime === undefined ? 0 : result.settings.onOffTimeSchedule.offTime.length);

                    for (var index = 0, i = 0; i < timerList.length; i++, index++) {
                        if (result.settings.onOffTimeSchedule.offTime[i] === null || result.settings.onOffTimeSchedule.offTime[i] === undefined) {
                            continue;
                        }
                        timerList[index] = {
                            hour: 0,
                            minute: 0,
                            week: 0
                        };

                        var tempWeek = result.settings.onOffTimeSchedule.offTime[i].day;
                        var weekToNum = 0;
                        for(var j = 0; j < tempWeek.length; j++){
                            if(tempWeek[j] === 'mon'){
                                weekToNum += 1;
                            }
                            if(tempWeek[j] === 'tue'){
                                weekToNum += 2;
                            }
                            if(tempWeek[j] === 'wed'){
                                weekToNum += 4;
                            }
                            if(tempWeek[j] === 'thu'){
                                weekToNum += 8;
                            }
                            if(tempWeek[j] === 'fri'){
                                weekToNum += 16;
                            }
                            if(tempWeek[j] === 'sat'){
                                weekToNum += 32;
                            }
                            if(tempWeek[j] === 'sun'){
                                weekToNum += 64;
                            }

                        }
                        timerList[index].hour = result.settings.onOffTimeSchedule.offTime[i].hour;
                        timerList[index].minute = result.settings.onOffTimeSchedule.offTime[i].minute;
                        timerList[index].week = weekToNum;
                    }

                    cbObj.timerList = timerList;

                    if (typeof successCallback === 'function') {
                        successCallback(cbObj);
                    }
                }
            },
            onFailure: function(result) {
                log("getOffTimerList: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGOTL", "Power.getOffTimerList returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.getOffTimerList Done");
    };

    /**
     * Turns on/off the display panel. This will only affect the panel, not the main power.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>displayMode</th><th>String</th><th><a href="Power.DisplayMode.html#constructor">Power.DisplayMode</a> </th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function setDisplayMode () {
     *   var options = {
     *      displayMode : Power.DisplayMode.DISPLAY_OFF
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.setDisplayMode(successCb, failureCb, options);
     * }
     * @since 1.0
     * @see
     * <a href="Power%23getPowerStatus.html">Power.getPowerStatus()</a><br>
     */
    Power.prototype.setDisplayMode = function(successCallback, errorCallback, options) {

        log("setDisplayMode: " + JSON.stringify(options));

        service.Request("luna://com.webos.service.commercial.scapadapter/power", {
            method: "setDisplayMode",
            parameters: {
                displayMode: options.displayMode
            },
            onSuccess: function(result) {
                log("setDisplayMode: On Success");

                if (result.returnValue === true) {
                    delete result.returnValue;
                    if (typeof successCallback === 'function') {
                        log("setDisplayMode: no need to do any action.");
                        successCallback();
                    }
                }
            },
            onFailure: function(result) {
                log("setDisplayMode: On Failure");
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PSDM", "Power.setDisplayMode returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.setDisplayMode Done");
    };

    /**
     * Executes power related command.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>powerCommand</th><th>String</th><th><a href="Power.PowerCommand.html#constructor">Power.PowerCommand</a> </th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with failure callback object as a parameter.</p>
     * @example
     * // Javascript code
     * function executePowerCommand () {
     *   var options = {
     *      powerCommand : Power.PowerCommand.REBOOT
     *   };
     *
     *   function successCb() {
     *      // Do something
     *   }
     *
     *   function failureCb(cbObject) {
     *      var errorCode = cbObject.errorCode;
     *      var errorText = cbObject.errorText;
     *      console.log ("Error Code [" + errorCode + "]: " + errorText);
     *   }
     *
     *   var power = new Power();
     *   power.executePowerCommand(successCb, failureCb, options);
     * }
     * @since 1.0
     * @see
     * <a href="Power%23getPowerStatus.html">Power.getPowerStatus()</a><br>
     */
    Power.prototype.executePowerCommand = function(successCallback, errorCallback, options) {

        log("executePowerCommand: " + JSON.stringify(options));

        service.Request("luna://com.webos.service.commercial.scapadapter/power", {
            method: "executePowerCommand",
            parameters: {
                powerCommand: options.powerCommand
            },
            onSuccess: function(result) {
                log("executePowerCommand: On Success");

                delete result.returnValue;
                if (typeof successCallback === 'function') {
                    successCallback();
                }
            },
            onFailure: function(result) {
                log("executePowerCommand: On Failure");

                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PEPM", "Power.executePowerCommand returns failure.");
                    errorCallback(result);
                }
            }
        });

        log("Power.executePowerCommand Done");

    };


    /**
     * Sets DPMWakeup, Each <a href="Power.DPMSignalType.html#constructor">DPMSignalType</a> has a set of predefined DPMSignalType properties.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>DPMSignalType</th><th>String</th><th><a href="Power.DPMSignalType.html#constructor">DPMSignalType</a></th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with a failure callback object as a parameter.</p>
     *
     * @example
     * // Javascript code
     * function setDPMWakeup () {
     *  var options = {};
     *  options.dpmSignalType = Power.DPMSignalType.CLOCK;
     *  function successCb(cbObject) {
     *     console.log("sucess");
     *     // Do something
     *  }
     *
     *  function failureCb(cbObject) {
     *     var errorCode = cbObject.errorCode;
     *     var errorText = cbObject.errorText;
     *     console.log ("Error Code [" + errorCode + "]: " + errorText);
     *  }
     *
     *  var power = new Power();
     *  power.setDPMWakeup(successCb, failureCb, options);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23getDPMWakeup.html">Power.getDPMWakeup()</a><br>
     */
    Power.prototype.setDPMWakeup = function(successCallback, errorCallback, options) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "setDPMWakeup",
            parameters : options,
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback();
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PSDW", "Power.setDPMWakeup returns failure.");
                    errorCallback(result);
                }
            }
        });
    };

    /**
     * Gets DPMWakeup Info, Each <a href="Power.DPMSignalType.html#constructor">DPMSignalType</a> has a set of predefined DPMSignalType properties.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>dpmSignalType</th><th>String</th><th><a href="Power.DPMSignalType.html#constructor">DPMSignalType</a></th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getDPMWakeup () {
     *  function successCb(cbObject) {
     *     console.log("cbObject : " + JSON.stringify(cbObject));
     *     // Do something
     *  }
     *
     *  function failureCb(cbObject) {
     *     var errorCode = cbObject.errorCode;
     *     var errorText = cbObject.errorText;
     *     console.log ("Error Code [" + errorCode + "]: " + errorText);
     *  }
     *
     *  var power = new Power();
     *  power.getDPMWakeup(successCb, failureCb);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23setDPMWakeup.html">Power.setDPMWakeup()</a><br>
     */
    Power.prototype.getDPMWakeup = function(successCallback, errorCallback) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "getDPMWakeup",
            parameters : {},
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback(result);
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGDW", "Power.getDPMWakeup returns failure.");
                    errorCallback(result);
                }
            }
        });
    };

    /**
     * Sets PMMode, Each <a href="Power.PMMode.html#constructor">PMMode</a> has a set of predefined PMMode properties.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>mode</th><th>String</th><th><a href="Power.PMMode.html#constructor">PMMode</a> (ScreenOffBacklight is only for outdoor model)</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with a failure callback object as a parameter.</p>
     *
     * @example
     * // Javascript code
     * function setPMMode () {
     *  var options = {};
     *  options.mode = "powerOff";
     *
     *  function successCb(cbObject) {
     *     console.log("sucess");
     *     // Do something
     *  }
     *
     *  function failureCb(cbObject) {
     *     var errorCode = cbObject.errorCode;
     *     var errorText = cbObject.errorText;
     *     console.log ("Error Code [" + errorCode + "]: " + errorText);
     *  }
     *
     *  var power = new Power();
     *  power.setPMMode(successCb, failureCb, options);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23getPMMode.html">Power.getPMMode()</a><br>
     */
    Power.prototype.setPMMode = function(successCallback, errorCallback, options) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "setPMMode",
            parameters : options,
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback();
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PSPM", "Power.setPMMode returns failure.");
                    errorCallback(result);
                }
            }
        });
    };

    /**
     * Gets PMMode Info, <a href="Power.PMMode.html#constructor">PMMode</a> has a set of predefined PMMode properties.
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>mode</th><th>String</th><th><a href="Power.PMMode.html#constructor">PMMode</a></th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getPMMode () {
     *      function successCb(cbObject) {
     *          console.log("cbObject : " + JSON.stringify(cbObject));
     *          // Do something
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *          }
     *      var power = new Power();
     *      power.getPMMode(successCb, failureCb);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23setPMMode.html">Power.setPMMode()</a><br>
     */
    Power.prototype.getPMMode = function(successCallback, errorCallback) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "getPMMode",
            parameters : {},
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback(result);
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGPM", "Power.getPMMode returns failure.");
                    errorCallback(result);
                }
            }
        });
    };
    /**
     * Sets PowerOnDelay
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th><th>Required</th></tr></thead>
     *   <tbody>
     *       <tr><th>delayTime</th><th>Number</th><th>Delay Time (0 ~ 250)</th><th>required</th></tr>
     *   </tbody>
     * </table>
     * </div>
     * @return <p>If the method is successfully executed, success callback function is called without a parameter.</br>
     * If an error occurs, failure callback function is called with a failure callback object as a parameter.</p>
     *
     * @example
     * // Javascript code
     * function setPowerOnDelay () {
     *      var options = {};
     *      options.delayTime = 20;
     *      function successCb(cbObject) {
     *          console.log("sucess");
     *          // Do something
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *          }
     *      var power = new Power();
     *      power.setPowerOnDelay(successCb, failureCb, options);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23getPowerOnDelay.html">Power.getPowerOnDelay()</a><br>
     */
    Power.prototype.setPowerOnDelay = function(successCallback, errorCallback, options) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "setPowerOnDelay",
            parameters : options,
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback();
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PSPD", "Power.setPowerOnDelay returns failure.");
                    errorCallback(result);
                }
            }
        });
    };

    /**
     * Gets PowerOnDelay
     * @class Power
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>delayTime</th><th>Number</th><th>Delay Time (0 ~ 250)</th></tr>
     *   </tbody>
     * </table>
     * </div>
     *
     * @example
     * // Javascript code
     * function getPowerOnDelay () {
     *      function successCb(cbObject) {
     *          console.log("cbObject : " + JSON.stringify(cbObject));
     *          // Do something
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *          }
     *      var power = new Power();
     *      power.getPowerOnDelay(successCb, failureCb);
     * }
     * @since 1.4
     * @see
     * <a href="Power%23setPowerOnDelay.html">Power.setPowerOnDelay()</a><br>
     */
    Power.prototype.getPowerOnDelay = function(successCallback, errorCallback) {
        service.Request("luna://com.webos.service.commercial.scapadapter/power/", {
            method : "getPowerOnDelay",
            parameters : {},
            onSuccess : function(result) {
                delete result.returnValue;
                if(typeof successCallback === 'function') {
                    successCallback(result);
                }
            },
            onFailure : function(result) {
                delete result.returnValue;
                if (typeof errorCallback === 'function') {
                    checkErrorCodeNText(result, "PGPD", "Power.getPowerOnDelay returns failure.");
                    errorCallback(result);
                }
            }
        });
    };

    /**
     * Gets On/Off Time Schedule
     * @class Time
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return {Object}
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>onOffTimeScheduleList[]</th><th>Array</th><th>on/Off Time schedule list</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeScheduleList[]._id</th><th>String</th><th>schedule id</th></tr>
     *       <tr><th>onOffTimeScheduleList[].day</th><th>String</th><th>day</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeScheduleList[].offTime.hour</th><th>Number</th><th>off Time hour</th></tr>
     *       <tr><th>onOffTimeScheduleList[].offTime.minute</th><th>Number</th><th>off Time minute</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeScheduleList[].onTime.hour</th><th>Number</th><th>on Time hour</th></tr>
     *       <tr><th>onOffTimeScheduleList[].onTime.minute</th><th>Number</th><th>on Time minute</th></tr>
     *   </tbody>
     * </table>
     *
     * @example
     * // Javascript code
     *  function getOnOffTimeSchedule () {
     *      function successCb(cbObject) {
     *          console.log("cbObject : " + JSON.stringify(cbObject));
     *          for (var i = cbObject.onOffTimeScheduleList.length-1; i>=0; i--) {
     *              console.log("onOffTimeScheduleList[" + i + "]._id : " + cbObject.onOffTimeScheduleList[i]._id);
     *              console.log("onOffTimeScheduleList[" + i + "].day : " + cbObject.onOffTimeScheduleList[i].day);
     *              console.log("onOffTimeScheduleList[" + i + "].offTime.hour : " + cbObject.onOffTimeScheduleList[i].offTime.hour);
     *              console.log("onOffTimeScheduleList[" + i + "].offTime.minute : " + cbObject.onOffTimeScheduleList[i].offTime.minute);
     *              console.log("onOffTimeScheduleList[" + i + "].onTime.hour : " + cbObject.onOffTimeScheduleList[i].onTime.hour);
     *              console.log("onOffTimeScheduleList[" + i + "].onTime.minute : " + cbObject.onOffTimeScheduleList[i].onTime.minute);
     *          }
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *      }
     *      var power = new Power();
     *      power.getOnOffTimeSchedule(successCb, failureCb);
     *  }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.getAllOnOffTimers instead.<br>
     * @since 1.5
     * @see
     * <a href="Power%23setOnOffTimeSchedule.html">Power.setOnOffTimeSchedule()</a><br>
     * <a href="Power%23unsetOnOffTimeSchedule.html">Power.unsetOnOffTimeSchedule()</a><br>
     */

    Power.prototype.getOnOffTimeSchedule = function(successCallback, errorCallback) {
        var ret = {};
        checkErrorCodeNText(ret, "DEPRECATED_API", "This function is deprecated.");
        errorCallback(ret);
    };

    /**
     * Sets On/Off time schedule
     * @class Time
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @param {Object} options
     * <div align=left>
     * <table class="hcap_spec" width=400>
     *   <thead><tr><th>Property</th><th>Type</th><th>Description</th></tr></thead>
     *   <tbody>
     *       <tr><th>onOffTimeSchedule[]</th><th>Array</th><th>on/Off Time Schedule list</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeSchedule[].day</th><th>String</th><th>day</th></tr>
     *       <tr><th>onOffTimeSchedule[].onTime.hour</th><th>Number</th><th>onTime hour</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeSchedule[].onTime.minute</th><th>Number</th><th>onTime minute</th></tr>
     *       <tr><th>onOffTimeSchedule[].offTime.hour</th><th>Number</th><th>offTime hour</th></tr>
     *       <tr class="odd"><tr><th>onOffTimeSchedule[].offTime.minute</th><th>Number</th><th>offTime minute</th></tr>
     *   </tbody>
     * </table>
     *
     * @example
     * // Javascript code
     *  function setOnOffTimeSchedule () {
     *      var params = {
     *           "onOffTimeSchedule":[
     *               {
     *                   "day":"fri",
     *                   "onTime":{
     *                       "hour":12,
     *                       "minute":0
     *                   },
     *                   "offTime":{
     *                       "hour":21,
     *                       "minute":0
     *                   }
     *               },
     *               {
     *                   "day":"mon",
     *                   "onTime":{
     *                       "hour":7,
     *                       "minute":0
     *                   },
     *                   "offTime":{
     *                       "hour":18,
     *                       "minute":0
     *                   }
     *               }
     *           ]
     *      }
     *      function successCb(cbObject) {
     *          console.log("success");
     *          // Do something
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *      }
     *      var power = new Power();
     *      power.setOnOffTimeSchedule(successCb, failureCb, params);
     *  }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.reserveOnOffTimer instead.<br>
     * @since 1.5
     * @see
     * <a href="Power%23getOnOffTimeSchedule.html">Power.getOnOffTimeSchedule()</a><br>
     * <a href="Power%23unsetOnOffTimeSchedule.html">Power.unsetOnOffTimeSchedule()</a><br>
     */

    Power.prototype.setOnOffTimeSchedule = function(successCallback, errorCallback, params) {
        var ret = {};
        checkErrorCodeNText(ret, "DEPRECATED_API", "This function is deprecated.");
        errorCallback(ret);
    };

    /**
     * Unset On/Off time schedule List
     * @class Time
     * @param {Function} successCallback success callback function.
     * @param {Function} errorCallback failure callback function.
     * @return {Object}
     * <div align=left>
     *
     * @example
     * // Javascript code
     *  function unsetOnOffTimeSchedule () {
     *      function successCb(cbObject) {
     *          console.log("success");
     *          // Do something
     *      }
     *      function failureCb(cbObject) {
     *          var errorCode = cbObject.errorCode;
     *          var errorText = cbObject.errorText;
     *          console.log ("Error Code [" + errorCode + "]: " + errorText);
     *      }
     *      var power = new Power();
     *      power.unsetOnOffTimeSchedule(successCb, failureCb);
     *  }
     * @deprecated
     * This function is deprecated since 1.6.0. Use Time.clearAllOnOffTimers instead.<br>
     * @since 1.5
     * @see
     * <a href="Power%23setOnOffTimeSchedule.html">Power.setOnOffTimeSchedule()</a><br>
     * <a href="Power%23getOnOffTimeSchedule.html">Power.getOnOffTimeSchedule()</a><br>
     */

    Power.prototype.unsetOnOffTimeSchedule = function(successCallback, errorCallback) {
        var ret = {};
        checkErrorCodeNText(ret, "DEPRECATED_API", "This function is deprecated.");
        errorCallback(ret);
    };

    return Power;
}());